@@ -62,12 +62,15 @@ def sync(loop, func, *args, timeout=None, **kwargs):
             timeout -= 1
             if timeout < 0:
                 raise FSTimeoutError
-    if isinstance(result[0], asyncio.TimeoutError):
+
+    return_result = result[0]
+    if isinstance(return_result, asyncio.TimeoutError):
         # suppress asyncio.TimeoutError, raise FSTimeoutError
-        raise FSTimeoutError
-    if isinstance(result[0], BaseException):
-        raise result[0]
-    return result[0]
+        raise FSTimeoutError from return_result
+    elif isinstance(return_result, BaseException):
+        raise return_result
+    else:
+        return return_result
 
 
 iothread = [None]  # dedicated fsspec IO thread
@@ -355,6 +358,26 @@ class AsyncFileSystem(AbstractFileSystem):
         else:
             return out[0]
 
+    async def _cat_ranges(self, paths, starts, ends, max_gap=None, **kwargs):
+        # TODO: on_error
+        if max_gap is not None:
+            # to be implemented in utils
+            raise NotImplementedError
+        if not isinstance(paths, list):
+            raise TypeError
+        if not isinstance(starts, list):
+            starts = [starts] * len(paths)
+        if not isinstance(ends, list):
+            ends = [starts] * len(paths)
+        if len(starts) != len(paths) or len(ends) != len(paths):
+            raise ValueError
+        return await asyncio.gather(
+            *[
+                self._cat_file(p, start=s, end=e, **kwargs)
+                for p, s, e in zip(paths, starts, ends)
+            ]
+        )
+
     async def _put(
         self, lpath, rpath, recursive=False, callback=_DEFAULT_CALLBACK, **kwargs
     ):
@@ -452,6 +475,9 @@ class AsyncFileSystem(AbstractFileSystem):
     async def _size(self, path):
         return (await self._info(path)).get("size", None)
 
+    async def _sizes(self, paths):
+        return await asyncio.gather(*[self._size(p) for p in paths])
+
     async def _exists(self, path):
         try:
             await self._info(path)
